'// Cinco Piedras (five stones)
'// Game logic written in 2005 by Casey Sprangel (www.ifxgroup.net)
'// Created using PowerBASIC for Windows (www.powerbasic.com)
'// Original board, grid and stone placement code written by Paul Dwyer, Network Engineer, Aussie in Tokyo
'--------------------------------------------------------------------
'// Things to change:
'// - buttons need color
'// - scrollbar needs color
'// - language selection needs pick list or graphic flags
'// - program needs icon
'// - winner makes first move in next game
'// - remember last selected language
'// - the "yes" "no" choice on the game restart needs to be language specific
'// - enable maximize button and scale graphics to fit box
'--------------------------------------------------------------------
'// Possible things to add:
'// + automated computer player (start easy and get harder)
'// + network play with another person including chat
'// + save game state on program close and restore same game on next program open
'// + save game history to a log file for later review
'// + load a game log file and see it play
'// + click on a point in the history log and see the board as it looked back then
'// + bitmapped graphics
'// + background music
'// + sound effects for stone placement, capture and game win
'// + "training mode" with active hilights showing possible captures, open 3 and open 4 groups
'// + "boss" mode to hide game from taskbar when not in the foreground
'// + ability to enter moves in traditional notation (as shown in history log)
'--------------------------------------------------------------------

#COMPILE EXE                                                          '// make this into an executable program
#INCLUDE "WIN32API.INC"                                               '// include the windows stuff for graphics

'--------------------------------------------------------------------
'// Static names for window controls
%IDC_GRAPHIC1       = 110
%IDC_PLAYER1_LABEL  = 111
%IDC_PLAYER2_LABEL  = 112
%IDC_COLOR_LABEL    = 113
%IDC_COLOR1_LABEL   = 114
%IDC_COLOR2_LABEL   = 115
%IDC_CAPTURE_LABEL  = 116
%IDC_CAPTURE1_LABEL = 117
%IDC_CAPTURE2_LABEL = 118
%IDC_SCORE_LABEL    = 119
%IDC_SCORE1_LABEL   = 120
%IDC_SCORE2_LABEL   = 121
%IDC_HISTORY_LABEL  = 122
%IDC_STATUS         = 123
%IDC_HISTORY        = 124
%IDC_LANG           = 125
%gone               = -3                                              '// flag to show that a stone has been captured (shows a shadow)

'// Language element identifiers.
'// Must be integer numbers starting with Zero.
'// Lang_Rules must be the last element with the highest number.
%lang_set         = 0
%lang_title       = 1
%lang_player      = 2
%lang_turn        = 3
%lang_color       = 4
%lang_green       = 5
%lang_red         = 6
%lang_blue        = 7
%lang_cyan        = 8
%lang_captures    = 9
%lang_points      = 10
%lang_history     = 11
%lang_bye         = 12
%lang_game_over   = 13
%lang_start_over  = 14
%lang_notice      = 15
%lang_not_here    = 16
%lang_wins        = 17
%lang_winner      = 18
%lang_tie         = 19
%lang_congrats    = 20
%lang_won5        = 21
%lang_won_capts   = 22
%lang_lang        = 23
%lang_yes         = 24
%lang_no          = 25

%lang_rules       = 26                                                '// this must be the one with the highest number!

'--------------------------------------------------------------------
'// These are all of the functions we have available
DECLARE FUNCTION count4(a AS STRING, c AS LONG) AS LONG
DECLARE FUNCTION makelang(c AS STRING) AS LONG
DECLARE FUNCTION newgame() AS LONG
DECLARE FUNCTION PlaceStar(X AS LONG, Y AS LONG) AS LONG
DECLARE FUNCTION PlaceStone(X AS LONG, Y AS LONG, Colour AS LONG) AS LONG
DECLARE FUNCTION plang(work AS LONG, player AS LONG, score AS LONG) AS STRING
DECLARE FUNCTION recount(player AS LONG) AS LONG
DECLARE FUNCTION think(x AS LONG, y AS LONG) AS LONG
DECLARE FUNCTION update_board() AS LONG

'--------------------------------------------------------------------
'// Define the global variables
GLOBAL backcolor AS LONG                                              '// the background when not on the playing board
GLOBAL board() AS LONG                                                '// holds the array of pieces
GLOBAL boardcolor AS LONG                                             '// the color of the board background
GLOBAL capture() AS LONG                                              '// holds the number of captures each player has made
GLOBAL gameover AS LONG                                               '// flag to show that the game is over
GLOBAL ghDlg AS DWORD                                                 '// the handle of our main dialog
GLOBAL history() AS STRING                                            '// holds the game action history
GLOBAL lang() AS STRING                                               '// holds the current language information (preloaded with english/spanish)
GLOBAL mousePointerLoc AS POINTAPI                                    '// the mouse pointer location
GLOBAL NextTurn AS LONG                                               '// holds the color of the next player
GLOBAL score() AS LONG                                                '// holds the total score for each player
GLOBAL ts() AS LONG                                                   '// count the number of temp points (like 4-in-a-row)
GLOBAL win5 AS LONG                                                   '// flag to show that we have a 5-in-a-row
GLOBAL winner AS LONG                                                 '// shows the player that won

'=====================================================================

FUNCTION plang(work AS LONG, player AS LONG, score AS LONG) AS STRING '// process a language file entry with parameters
  LOCAL buf AS STRING
  LOCAL i AS LONG
  buf = lang(work)                                                    '// make a copy of the language string so we can modify
  IF INSTR(buf, "@") > 0 THEN                                         '// is there a player parameter?
    REPLACE "@" WITH FORMAT$(player) IN buf                           '// put that in our string
    IF INSTR(buf, "#") > 0 THEN                                       '// is there a score parameter?
      REPLACE "#" WITH FORMAT$(score) IN buf
    END IF
  ELSE                                                                '// no at sign?
    IF INSTR(buf, "#") > 0 THEN                                       '// is there a player parameter?
      REPLACE "#" WITH FORMAT$(player) IN buf
    END IF
  END IF
  i = INSTR(UCASE$(buf), "\N")                                        '// find first newline
  DO WHILE i > 0                                                      '// loop until we find them all
    buf = LEFT$(buf, i -1) + $CRLF + $CRLF + MID$(buf, i + 2)         '// turn newline markers into actual line breaks
    i = INSTR(UCASE$(buf), "\N")                                      '// find next newline (if there is one)
  LOOP
  FUNCTION = buf
END FUNCTION

'=====================================================================

FUNCTION makelang(c AS STRING) AS LONG                                '// populate our language table with text
  SELECT CASE UCASE$(c)                                               '// what language has been requested?
    CASE "EN"                                                         '// English
      lang(%lang_set) = "EN"
      lang(%lang_yes) = "Yes"
      lang(%lang_no) = "No"
      lang(%lang_title) = "Five Pebbles"
      lang(%lang_player) = "Player"
      lang(%lang_turn) = "Player # turn."
      lang(%lang_color) = "Colour"
      lang(%lang_green) = "Green"
      lang(%lang_red) = "Red"
      lang(%lang_blue) = "Blue"
      lang(%lang_cyan) = "Cyan"
      lang(%lang_captures) = "Captures"
      lang(%lang_points) = "Points"
      lang(%lang_history) = "History"
      lang(%lang_bye) = "&Bye"
      lang(%lang_game_over) = "Game Over"
      lang(%lang_start_over) = "Do you want to start over?"
      lang(%lang_notice) = "Notice"
      lang(%lang_not_here) = "You can't put your stone there. Try someplace else."
      lang(%lang_wins) = "Player # wins."
      lang(%lang_winner) = "Winner!"
      lang(%lang_tie) = "Tie game."
      lang(%lang_congrats) = "Congratulations"
      lang(%lang_won5) = "Player # has five in a row."
      lang(%lang_won_capts) = "Player @ has won # captures!\nGame over."
      lang(%lang_lang) = "&Language"
      lang(%lang_rules) = "This is a game of skill that is like a mixture of tic-tac-toe and chess.\n" + _
                    "Each player has game pieces, called stones, that they add to the board. " + _
                    "The players take turns adding stones on the intersections anywhere on the board including the edge.\n" + _
                    "The first player must put a stone in the center of the playing board. " + _
                    "This is automatically done for you.\n" + _
                    "If two of your stones surround two of your opponant's touching stones to form a straight line in any direction, that is considered a capture and your opponenant's stones will be removed.\n" + _
                    "The goal is to get five consecutive stones in a row in a straight line in any direction or five captures.\n" + _
                    "You will not be captured by putting your stones between those of your opponant.\n" + _
                    "The player that reached the goal may not be the final winner. " + _
                    "The score is used to find the winner. " + _
                    "One point is given for each capture. " + _
                    "One point is given for every four-in-a-row of your stones that remain on the board at the end of the game. " + _
                    "Five points are given for a five-in-a-row."

    CASE "NL"                                                         '// Dutch
      lang(%lang_set) = "NL"
      lang(%lang_yes) = "Yes"
      lang(%lang_no) = "No"
      lang(%lang_title) = "Vijf Kiezelstenen"
      lang(%lang_player) = "Speler"
      lang(%lang_turn) = "Speler # draai."
      lang(%lang_color) = "kleur"
      lang(%lang_green) = "Groen"
      lang(%lang_red) = "Rood"
      lang(%lang_blue) = "Blauw"
      lang(%lang_cyan) = "Cyaan"
      lang(%lang_captures) = "Vangt"
      lang(%lang_points) = "Punten"
      lang(%lang_history) = "Geschiedenis"
      lang(%lang_bye) = "Tot &ziens"
      lang(%lang_game_over) = "Spel over"
      lang(%lang_start_over) = "Wilt u over beginnen?"
      lang(%lang_notice) = "Bericht"
      lang(%lang_not_here) = "U kunt niet uw steen daar zetten. Probeer someplace anders."
      lang(%lang_wins) = "Speler # wint."
      lang(%lang_winner) = "Winnaar!"
      lang(%lang_tie) = "Het spel van de band."
      lang(%lang_congrats) = "Gelukwensen"
      lang(%lang_won5) = "Speler # heeft vijf in een rij."
      lang(%lang_won_capts) = "Speler @ heeft # vangt gewonnen!\nSpel over."
      lang(%lang_lang) = "&Taal"
      lang(%lang_rules) = "Dit is een spel van vaardigheid dat als een mengsel van tic-tac-teen en schaak is.\n" + _
                    "Elke speler heeft spelstukken, genoemd stenen, die zij aan de raad toevoegen. " + _
                    "De spelers nemen draaien overal toevoegend stenen op de kruisingen op de raad met inbegrip van de rand.\n" + _
                    "De eerste speler moet een steen in het centrum van de speelraad zetten. " + _
                    "Dit wordt automatisch gedaan voor u.\n" + _
                    "Als twee van uw stenen twee van uw opponant wat betreft stenen omringen om een rechte lijn in om het even welke richting te vormen, wordt dat beschouwd als een vangst en uw opponenant stenen zullen worden verwijderd.\n" + _
                    "Het doel is vijf opeenvolgende stenen in een rij in een rechte lijn in om het even welke richting te krijgen of vijf vangen.\n" + _
                    "U zult niet gevangen worden door uw stenen tussen die van uw opponant te zetten.\n" + _
                    "De speler die het doel bereikte kan niet de definitieve winnaar zijn. " + _
                    "De score wordt gebruikt om de winnaar te vinden. " + _
                    "n punt wordt gegeven voor elke vangst. " + _
                    "n punt wordt gegeven voor elke vier-in-a-rij van uw stenen die op de raad aan het eind van het spel blijven. " + _
                    "Vijf punten worden gegeven voor een vijf-in-a-rij."
    CASE ELSE                                                         '// Spanish (the default language)
      lang(%lang_set) = "ES"
      lang(%lang_yes) = "Si"
      lang(%lang_no) = "No"
      lang(%lang_title) = "Cinco Piedras"
      lang(%lang_player) = "Jugador"
      lang(%lang_turn) = "Jugador # siguiente."
      lang(%lang_color) = "Color"
      lang(%lang_green) = "Verde"
      lang(%lang_red) = "Rojo"
      lang(%lang_blue) = "Azul"
      lang(%lang_cyan) = "Cinico"
      lang(%lang_captures) = "Capturas"
      lang(%lang_points) = "Puntos"
      lang(%lang_history) = "Historia"
      lang(%lang_bye) = "&Adis"
      lang(%lang_game_over) = "Juego encima."
      lang(%lang_start_over) = "Usted desea comenzar encima?"
      lang(%lang_notice) = "Aviso"
      lang(%lang_not_here) = "Usted no puede poner su piedra all. Intento en alguna parte."
      lang(%lang_wins) = "El jugador # ha ganado!"
      lang(%lang_winner) = "Ganador!"
      lang(%lang_tie) = "Cuenta del lazo."
      lang(%lang_congrats) = "Felicitaciones"
      lang(%lang_won5) = "El jugador # tiene cinco en una fila."
      lang(%lang_won_capts) = "El jugador @ ha ganado # capturas!\nJuego encima."
      lang(%lang_lang) = "&Lengua"
      lang(%lang_rules) = "ste es un juego de la habilidad que es como una mezcla del tic-tac-toe del pie y del ajedrez.\n" + _
                    "Cada jugador tiene pedazos del juego, llamados las piedras, que agregan al tablero. " + _
                    "Los jugadores toman las vueltas que agregan piedras en las intersecciones dondequiera en el tablero incluyendo el borde.\n" + _
                    "El primer jugador debe poner una piedra en el centro del tablero que juega. " + _
                    "Esto se hace automticamente para usted.\n" + _
                    "Si dos de sus piedras rodean dos de las piedras de tacto de sus opponant para formar una lnea recta en cualquier direccin, eso se considera una captura y las piedras de sus opponenant sern quitadas.\n" + _
                    "La meta es conseguir cinco piedras consecutivas en una fila en una lnea recta en cualquier direccin o cinco capturas.\n" + _
                    "Colocando sus piedras entre las de su opponant le no capturar.\n" + _
                    "El jugador que alcanz la meta puede no ser el ganador final. " + _
                    "La cuenta se utiliza para encontrar al ganador. " + _
                    "Un punto se da para cada captura. " + _
                    "Un punto se da para cada cuatro-en-uno-fila de sus piedras que permanezcan en el tablero en el extremo del juego. " + _
                    "Cinco puntos se dan para una cinco-en-uno-fila."
  END SELECT
END FUNCTION

'=====================================================================

FUNCTION newgame() AS LONG                                            '// get everything [re]set up for a new game to start
  REGISTER x AS LONG
  REGISTER y AS LONG
  REDIM board(19,19)                                                  '// start with a blank board
  REDIM capture(2)                                                    '// start with no captures
  REDIM score(2)                                                      '// start with no score
  REDIM ts(2)                                                         '// start with no temp score
  REDIM history(0)                                                    '// no history
  winner = 0
  win5 = 0                                                            '// reset flag
  CONTROL SET COLOR ghDlg, %IDC_CAPTURE1_LABEL, %BLACK, boardcolor    '// restore normal colors
  CONTROL SET COLOR ghDlg, %IDC_CAPTURE2_LABEL, %BLACK, boardcolor
  CONTROL SET TEXT ghDlg, %IDC_CAPTURE1_LABEL, FORMAT$(capture(1))    '// show the captures
  CONTROL SET TEXT ghDlg, %IDC_CAPTURE2_LABEL, FORMAT$(capture(2))
  CONTROL SET TEXT ghDlg, %IDC_SCORE1_LABEL, FORMAT$(score(1))        '// show the scores
  CONTROL SET TEXT ghDlg, %IDC_SCORE2_LABEL, FORMAT$(score(2))
  CONTROL SET COLOR ghDlg, %IDC_STATUS, %WHITE, %RED
  CONTROL SET TEXT ghDlg, %IDC_STATUS, ""
'  IF gameover THEN                                                    '// is this starting over from a previous game?
    CONTROL SET TEXT ghDlg, %IDC_HISTORY, "1:" + $TAB + "0,0"         '// show the action history log
    RANDOMIZE                                                         '// seed the random number generator
    ts(1) = 4                                                         '// start our loop count at 4
    DO WHILE ts(1) > -2                                               '// start the loop
      FOR y = 1 TO 19                                                 '// walk the board
        FOR x = 1 TO 19
          board(x, y) = RND(0,4)                                      '// make some interesting stuff
          PlaceStone(x, y, board(x, y))                               '// put stone on the board
        NEXT
      NEXT
      GRAPHIC REDRAW                                                  '// update the display
      SLEEP 50                                                        '// wait a while so the user can see what we did
      DECR ts(1)                                                      '// lower our loop counter
    LOOP
    gameover = 0
'  END IF
  FOR x = 1 TO 19                                                     '// clean the board for play
    FOR y = 1 TO 19
      board(x, y) = -1                                                '// make everything an open place
    NEXT
  NEXT
  board(10,10) = 1                                                    '// player 1 starts here
  history(0) = "0,0"                                                  '// record the starting move
  NextTurn = 2                                                        '// player 2 has the next move
  CONTROL SET TEXT ghDlg, %IDC_STATUS, "<-- " + plang(%lang_turn, NextTurn, 0)
END FUNCTION

'=====================================================================

FUNCTION update_board() AS LONG                                       '// make all memory changes show up on the screen
  REGISTER x AS LONG
  REGISTER y AS LONG
  REGISTER i AS LONG
  LOCAL buf AS STRING

  '// Clear the board and set background
  GRAPHIC CLEAR boardcolor, 0                                         '// start with a blank board
  GRAPHIC FONT "Times New Roman", 72, 3                               '// select font in italics
  GRAPHIC TEXT SIZE LEFT$(lang(%lang_title), INSTR(lang(%lang_title) + " ", " ")-1) TO x, y '// find the size of our text in pixels
  GRAPHIC SET POS (15, 400 - y)                                       '// move the pointer to the bottom of the board
  GRAPHIC COLOR backcolor, boardcolor                                 '// make color for printing
  GRAPHIC PRINT LEFT$(lang(%lang_title), INSTR(lang(%lang_title) + " ", " ")-1) '// show our text
  IF winner <> 0 THEN                                                 '// do we have a winner?
    IF winner = -1 THEN                                               '// is it a tie game?
      buf = lang(%lang_tie)                                           '// get the "tie" langauge entry
    ELSE                                                              '// no, a player won
      buf = lang(%lang_winner)                                        '// get "winner" language entry
    END IF
    i = 72                                                            '// start with a large point size
    DO
      GRAPHIC FONT "Times New Roman", i, 1                            '// select font in bold
      GRAPHIC TEXT SIZE buf TO x, y                                   '// find the size of our text in pixels
      IF x < 400 THEN                                                 '// will this text fit on the screen?
        EXIT LOOP                                                     '// leave the loop
      END IF
      DECR i                                                          '// lower the point size by one and try again
    LOOP
    '// text shadow
    GRAPHIC SET POS ((400 - x + 4) / 2, (400 - y + 4) / 2)            '// move the pointer to the middle of the board with shadow offset
    GRAPHIC COLOR %BLACK, -2                                          '// make color for printing with transparent background
    GRAPHIC PRINT buf;                                                '// show our text
    GRAPHIC SET POS ((400 - x) / 2, (400 - y) / 2)                    '// move the pointer to the middle of the board
    SELECT CASE winner                                                '// who won the game?
      CASE 1                                                          '// player 1?
        i = %GREEN                                                    '// their stone color is green
      CASE 2                                                          '// palyer 2?
        i = %RED                                                      '// their stone color is red
      CASE ELSE
        i = %BLUE
    END SELECT
    GRAPHIC COLOR i, -2                                               '// make color for printing with transparent background
    GRAPHIC PRINT buf;                                                '// show our text
  END IF

  '// Draw Board Lines
  GRAPHIC COLOR %BLACK, boardcolor                                    '// make dark lines
  FOR x = 1 TO 19
    GRAPHIC LINE (x * 20, 20 )-(x * 20, 380)                          '// draw across the board
  NEXT
  FOR y = 1 TO 19
    GRAPHIC LINE ( 20, y * 20 )-(380, y * 20)                         '// draw down the board
  NEXT
  '// Place navigation markers
  IF board(4,4) = -1 THEN board(4,4) = -2                             '// only if it is blank
  IF board(4,10) = -1 THEN  board(4,10) = -2
  IF board(4,16) = -1 THEN  board(4,16) = -2
  IF board(7,7) = -1 THEN board(7,7) = -2
  IF board(7,13) = -1 THEN board(7,13) = -2
  IF board(10,4) = -1 THEN  board(10,4) = -2
  IF board(10,10) = -1 THEN  board(10,10) = -2
  IF board(10,16) = -1 THEN  board(10,16) = -2
  IF board(13,7) = -1 THEN board(13,7) = -2
  IF board(13,13) = -1 THEN board(13,13) = -2
  IF board(16,4) = -1 THEN  board(16,4) = -2
  IF board(16,10) = -1 THEN  board(16,10) = -2
  IF board(16,16) = -1 THEN  board(16,16) = -2
  FOR x = 1 TO 19                                                     '// put all of the elements back on the board
    FOR y = 1 TO 19
      IF board(x, y) = -2 THEN                                        '// is this a star?
        PlaceStar(x, y)                                               '// put the star on the board
      ELSE                                                            '// this must be a piece
        IF board(x,y) > -1 OR board(x,y) = %gone THEN                 '// is there a piece here?
          PlaceStone(x, y, board(x, y))                               '// put stone on the board
          IF board(x,y) < -3 THEN                                     '// special case for captured stones
            INCR board(x, y)                                          '// turn this into a blank on next move
          ELSEIF board(x,y) = %gone THEN                               '// is this a captured shadow?
            board(x, y) = -1
          END IF
        END IF
      END IF
    NEXT y
  NEXT x
  GRAPHIC REDRAW                                                      '// update the display with our buffered changes (looks instant to user)
END FUNCTION
'====================================================================

FUNCTION PBMAIN() AS LONG                                             '// program starts here
  DIM lang(%lang_rules) AS STRING                                     '// make room for the language strings
  makelang("ES")                                                      '// build the language information for Spanish
  backcolor = RGB(&Hee,&Hdd,&H99)                                     '// the background when not on the playing board
  boardcolor = RGB(&HFF,&Hee,&Haa)                                    '// the playing board background color
  'Create the board
  DIALOG NEW PIXELS, 0, lang(%lang_title),,, 600, 400, %WS_CAPTION OR %WS_SYSMENU OR %WS_MINIMIZEBOX, 0 TO ghDlg
  DIALOG SET COLOR ghDlg, %BLACK, backcolor
  CONTROL ADD GRAPHIC, ghDlg, %IDC_GRAPHIC1,"", 0, 0, 400, 400, %WS_BORDER OR %SS_NOTIFY
  GRAPHIC ATTACH ghDlg, %IDC_GRAPHIC1, REDRAW
  CONTROL ADD LINE, ghDlg, -1, "", 402, 2, 190, 85, %SS_BLACKFRAME
  CONTROL ADD LABEL, ghDlg, %IDC_PLAYER1_LABEL, lang(%lang_player) + " 1",  460, 4, 60, 15, %SS_CENTER 'OR %SS_SUNKEN
  CONTROL SET COLOR ghDlg, %IDC_PLAYER1_LABEL, %BLACK, backcolor
  CONTROL ADD LABEL, ghDlg, %IDC_PLAYER2_LABEL, lang(%lang_player) + " 2",  525, 4, 60, 15, %SS_CENTER 'OR %SS_SUNKEN
  CONTROL SET COLOR ghDlg, %IDC_PLAYER2_LABEL, %BLACK, backcolor
  CONTROL ADD LABEL, ghDlg, %IDC_COLOR_LABEL, lang(%lang_color),      405, 22, 50, 15, %SS_RIGHT 'OR %SS_SUNKEN
  CONTROL SET COLOR ghDlg, %IDC_COLOR_LABEL, %BLACK, backcolor
  CONTROL ADD LABEL, ghDlg, %IDC_COLOR1_LABEL, lang(%lang_green),      460, 22, 60, 15, %SS_CENTER OR %SS_SUNKEN
  CONTROL SET COLOR ghDlg, %IDC_COLOR1_LABEL, %BLACK, %GREEN
  CONTROL ADD LABEL, ghDlg, %IDC_COLOR2_LABEL, lang(%lang_red),        525, 22, 60, 15, %SS_CENTER OR %SS_SUNKEN
  CONTROL SET COLOR ghDlg, %IDC_COLOR2_LABEL, %WHITE, %RED
  CONTROL ADD LABEL, ghDlg, %IDC_CAPTURE_LABEL, lang(%lang_captures), 405, 42, 50, 15, %SS_RIGHT 'OR %SS_SUNKEN
  CONTROL SET COLOR ghDlg, %IDC_CAPTURE_LABEL, %BLACK, backcolor
  CONTROL ADD TEXTBOX, ghDlg, %IDC_CAPTURE1_LABEL, "0",      460, 42, 60, 15, %SS_CENTER OR %ES_READONLY, %WS_EX_STATICEDGE
  CONTROL SET COLOR ghDlg, %IDC_CAPTURE1_LABEL, %BLACK, boardcolor
  CONTROL ADD TEXTBOX, ghDlg, %IDC_CAPTURE2_LABEL, "0",      525, 42, 60, 15, %SS_CENTER OR %ES_READONLY, %WS_EX_STATICEDGE
  CONTROL SET COLOR ghDlg, %IDC_CAPTURE2_LABEL, %BLACK, boardcolor
  CONTROL ADD LABEL, ghDlg, %IDC_SCORE_LABEL, lang(%lang_points),      405, 62, 50, 15, %SS_RIGHT 'OR %SS_SUNKEN
  CONTROL SET COLOR ghDlg, %IDC_SCORE_LABEL, %BLACK, backcolor
  CONTROL ADD TEXTBOX, ghDlg, %IDC_SCORE1_LABEL, "0",        460, 62, 60, 15, %SS_CENTER OR %ES_READONLY, %WS_EX_STATICEDGE
  CONTROL SET COLOR ghDlg, %IDC_SCORE1_LABEL, %BLACK, boardcolor
  CONTROL ADD TEXTBOX, ghDlg, %IDC_SCORE2_LABEL, "0",        525, 62, 60, 15, %SS_CENTER OR %ES_READONLY, %WS_EX_STATICEDGE
  CONTROL SET COLOR ghDlg, %IDC_SCORE2_LABEL, %BLACK, boardcolor
  CONTROL ADD LABEL, ghDlg, %IDC_HISTORY_LABEL, lang(%lang_history), 405, 94, 95, 15', %SS_SUNKEN
  CONTROL SET COLOR ghDlg, %IDC_HISTORY_LABEL, %BLACK, backcolor
  CONTROL ADD BUTTON, ghDlg, %IDC_LANG, lang(%lang_lang), 533, 90, 60, 19
  CONTROL ADD TEXTBOX, ghDlg, %IDC_HISTORY, plang(%lang_rules,0,0) + $CRLF + $CRLF + "(www.ifxgroup.net)", 404, 110, 192, 267 , %ES_MULTILINE OR %ES_READONLY OR %WS_VSCROLL, %WS_EX_STATICEDGE
  CONTROL SET COLOR ghDlg, %IDC_HISTORY, %BLACK, backcolor
  CONTROL ADD LABEL, ghDlg, %IDC_STATUS, "<-- " + plang(%lang_turn, 2, 0), 404, 380, 140, 18, %SS_SUNKEN
  CONTROL ADD BUTTON, ghDlg, %IDCANCEL, lang(%lang_bye), 545, 378, 50, 19
  GRAPHIC COLOR RGB(&H00,&H00,&H00), boardcolor

  newgame                                                             '// set the board up to start a new game

'// The following block is for testing the stone capture logic in the 8 possible directions.
$IF 0
  board(10,9) = 1  :board(10,8) = 1  :board(10,7) = 2                 '// north
  board(9,11) = 1  :board(8,12) = 1  :board(7,13) = 2                 '// north east
  board(9,10) = 1  :board(8,10) = 1  :board(7,10) = 2                 '// east
  board(11,9) = 1  :board(12,8) = 1  :board(13,7) = 2                 '// south east
  board(10,11) = 1 :board(10,12) = 1 :board(10,13) = 2                '// south
  board(11,11) = 1 :board(12,12) = 1 :board(13,13) = 2                '// south west
  board(11,10) = 1 :board(12,10) = 1 :board(13,10) = 2                '// west
  board(9,9) = 1   :board(8,8) = 1   :board(7,7) = 2                  '// north west
  board(10,10) = -1                                                   '// remove the default first player stone
$ENDIF

  update_board                                                        '// make sure our changes show on the board
  DIALOG SHOW MODAL ghDlg CALL DlgProc
END FUNCTION

'====================================================================

CALLBACK FUNCTION DlgProc() AS LONG                                   '// this handles all of the user interaction
  LOCAL sBmpMem AS STRING
  LOCAL DialogLocX AS LONG
  LOCAL DialogLocy AS LONG
  LOCAL i AS LONG
  SELECT CASE CBMSG
    CASE %WM_COMMAND
      SELECT CASE CBCTL
        CASE %IDCANCEL                                                '// cancel button
          IF CBCTLMSG = %BN_CLICKED THEN                              '// did user click it? (escape key works here too)
            GRAPHIC WINDOW END                                        '// destroy the graphic (stops crashing on multiple use)
            DIALOG END CBHNDL                                         '// destroy the dialog window
          END IF
        CASE %IDC_LANG
          IF CBCTLMSG = %STN_CLICKED THEN                             '// they clicked it!
            IF lang(%lang_set) = "EN" THEN
              makelang("ES")                                          '// make spanish
            ELSEIF lang(%lang_set) = "ES" THEN
              makelang("NL")                                          '// make dutch
            ELSE
              makelang("EN")                                          '// make english
            END IF
            DIALOG SET TEXT ghDlg, lang(%lang_title)                  '// update all of the display text with the new language
            CONTROL SET TEXT ghDlg, %IDC_PLAYER1_LABEL, lang(%lang_player) + " 1"
            CONTROL SET TEXT ghDlg, %IDC_PLAYER2_LABEL, lang(%lang_player) + " 2"
            CONTROL SET TEXT ghDlg, %IDC_COLOR_LABEL, lang(%lang_color)
            CONTROL SET TEXT ghDlg, %IDC_COLOR1_LABEL, lang(%lang_green)
            CONTROL SET TEXT ghDlg, %IDC_COLOR2_LABEL, lang(%lang_red)
            CONTROL SET TEXT ghDlg, %IDC_CAPTURE_LABEL, lang(%lang_captures)
            CONTROL SET TEXT ghDlg, %IDC_SCORE_LABEL, lang(%lang_points)
            CONTROL SET TEXT ghDlg, %IDC_HISTORY_LABEL, lang(%lang_history)
            IF NOT gameover THEN
              CONTROL SET TEXT ghDlg, %IDC_HISTORY, plang(%lang_rules,0,0) + $CRLF + $CRLF + "(www.ifxgroup.net)"
            END IF
            CONTROL SET TEXT ghDlg, %IDC_LANG, lang(%lang_lang)
            CONTROL SET TEXT ghDlg, %IDCANCEL, lang(%lang_bye)
            IF gameover THEN                                          '// is the game over?
              IF winner > 0 THEN                                      '// did a player win?
                CONTROL SET TEXT ghDlg, %IDC_STATUS, plang(%lang_wins, winner, 0)
              ELSE                                                    '// no, it was a tie game
                CONTROL SET TEXT ghDlg, %IDC_STATUS, lang(%lang_tie)
              END IF
            ELSE                                                      '// no, we are still playing the game
              CONTROL SET TEXT ghDlg, %IDC_STATUS, "<-- " + plang(%lang_turn, NextTurn, 0)
            END IF
            update_board
          END IF
        CASE %IDC_GRAPHIC1                                            '// did user do something to our graphic?
          IF CBCTLMSG = %STN_CLICKED THEN                             '// they clicked it!
            IF gameover THEN                                          '// is the game over flag set?
              i = MSGBOX(lang(%lang_start_over), %MB_YESNO OR %MB_ICONQUESTION OR %MB_TASKMODAL  , lang(%lang_game_over))
              IF i = %IDYES THEN                                      '// did user answer "yes"?
                newgame                                               '// reset the board and counters
                update_board                                          '// show the changes
              END IF
              EXIT FUNCTION                                           '// leave, we are done here
            END IF
            GETCURSORPOS mousePointerLoc                              '// get mouse coords
            DIALOG GET LOC ghDlg TO DialogLocX, DialogLocY            '// get dialog coords
            mousePointerLoc.x = mousePointerLoc.x - DialogLocX - 3    '// normalize coords for graphics control
            mousePointerLoc.y = mousePointerLoc.y - DialogLocY - 22
            mousePointerLoc.x = mousePointerLoc.x / 20                '// make into board points
            mousePointerLoc.y = mousePointerLoc.y / 20
            IF mousePointerLoc.x < 1 OR mousePointerLoc.x > 19 OR mousePointerLoc.y < 1 OR mousePointerLoc.y > 19 THEN '// illegal move?
              EXIT FUNCTION                                           '// leave without changing anything
            END IF
            IF board(mousePointerLoc.x, mousePointerLoc.y) < 0 THEN   '// is this place empty on the board?
              PlaceStone(mousePointerLoc.x, mousePointerLoc.y, NextTurn) '// put stone on board
              GRAPHIC REDRAW                                          '// allow graphic change to show
              i = UBOUND(history()) + 1                               '// find the end of our history list
              REDIM PRESERVE history(i)                               '// add one to the end of the list table
              history(i) = FORMAT$(mousePointerLoc.x - 10) + ", " + FORMAT$(mousePointerLoc.y - 10) '// add the history entry for this move
              think(mousePointerLoc.x, mousePointerLoc.y)             '// figure out what needs to be done next (captures, update score, etc.)
              IF gameover THEN                                        '// did this win the game?
                EXIT FUNCTION                                         '// we are done here, leave
              END IF
              IF NextTurn = 1 THEN                                    '// did player 1 just go?
                NextTurn = 2                                          '// it is now player 2 turn
                CONTROL SET COLOR ghDlg, %IDC_STATUS, %WHITE, %RED    '// change to show player color
              ELSE                                                    '// it was player 2 that just added a stone
                NextTurn = 1                                          '// it is now player 1 turn
               CONTROL SET COLOR ghDlg, %IDC_STATUS, %BLACK, %GREEN   '// show their color
              END IF
              CONTROL SET TEXT ghDlg, %IDC_STATUS, "<-- " + plang(%lang_turn, NextTurn, 0)
            ELSE                                                      '// not empty?
              MSGBOX lang(%lang_not_here), %MB_OK OR %MB_ICONINFORMATION OR %MB_TASKMODAL, lang(%lang_notice)
            END IF
          END IF
      END SELECT
  END SELECT
END FUNCTION

'=====================================================================

FUNCTION PlaceStone(x AS LONG, y AS LONG, c AS LONG) AS LONG          '// adds or removes a stone (the graphic work)
  LOCAL CalcedX AS LONG                                               '// our graphic location (pixels) of the piece on the board
  LOCAL CalcedY AS LONG
  LOCAL colour AS LONG                                                '// the normal color of the stone
  LOCAL Lcolour AS LONG                                               '// the low (dark) color of the stone
  LOCAL Pi AS DOUBLE
  CalcedX = ((x - 1) * 20) + 10                                       '// find screen pixel offset
  CalcedY = ((y - 1) * 20) + 10                                       '// find screen pixel offset
  board(x, y) = c                                                     '// record this player's move
  IF c < 0 THEN                                                       '// remove a stone?
    GRAPHIC ELLIPSE (CalcedX, CalcedY)-(CalcedX + 20, CalcedY + 20), boardcolor, backcolor, 0 '// erase the stone, but leave shadow
    GRAPHIC LINE (CalcedX , CalcedY + 10 )-(CalcedX + 20, CalcedY + 10), %BLACK '// restore the lines
    GRAPHIC LINE (CalcedX + 10, CalcedY )-(CalcedX + 10, CalcedY + 20), %BLACK
    SELECT CASE FORMAT$(x) + "-" + FORMAT$(y)                         '// is this a special marker location?
      CASE "4-4", "4-10", "4-16", "7-7", "7-13", "10-4", "10-10", "10-16", "13-7", "13-13", "16-4", "16-10", "16-16"
        GRAPHIC ELLIPSE (CalcedX + 8, CalcedY + 8)-(CalcedX + 13, CalcedY + 13), %BLACK, boardcolor, 0
    END SELECT
  ELSE                                                                '// place a stone
    SELECT CASE c
      CASE 0                                                          '// winner!
        colour = %BLACK
        Lcolour = %BLACK
      CASE 1                                                          '// player 1 (green)
        colour = RGB(0, 255, 0)
        Lcolour = RGB(0, 196, 0)
      CASE 2                                                          '// player 2 (red)
        colour = RGB(255, 0, 0)
        Lcolour = RGB(196, 0, 0)
      CASE 3                                                          '// player 3 (blue)
        colour = RGB(0, 0, 255)
        Lcolour = RGB(0, 0, 196)
      CASE 4                                                          '// player 4 (cyan)
        colour = RGB(0, 255, 255)
        Lcolour = RGB(0, 196, 196)
    END SELECT
    GRAPHIC ELLIPSE (CalcedX, CalcedY)-(CalcedX + 20, CalcedY + 20), %BLACK, colour, 0
    Pi = 4 * ATN(1)                                                   '// calculate Pi
    GRAPHIC PIE (CalcedX + 2, CalcedY + 2)-(CalcedX + 18, CalcedY + 18), Pi * 1.2, 0, Lcolour, Lcolour  '// stone lowlight ("under" the stone)
    GRAPHIC LINE (CalcedX + IIF(x = 1, 10, 0), CalcedY + 10)-(CalcedX + 20 + IIF(x = 19, -10, 0), CalcedY + 10), RGB(64, 64, 64) '// make "transparent" lines
    GRAPHIC LINE (CalcedX + 10, CalcedY + IIF(y = 1, 10, 0))-(CalcedX + 10, CalcedY + 20 + IIF(y = 19, -10, 0)), RGB(64, 64, 64)
    SELECT CASE FORMAT$(x) + "-" + FORMAT$(y)                         '// is this a special marker location?
      CASE "4-4", "4-10", "4-16", "7-7", "7-13", "10-4", "10-10", "10-16", "13-7", "13-13", "16-4", "16-10", "16-16"
        GRAPHIC ELLIPSE (CalcedX + 8, CalcedY + 8)-(CalcedX + 13, CalcedY + 13), RGB(64, 64, 64), colour, 0
    END SELECT
    GRAPHIC PIE (CalcedX +4, CalcedY +4)-(CalcedX + 12, CalcedY + 12), Pi/2,  Pi, %WHITE, %WHITE '// stone highlight
  END IF
END FUNCTION

'=====================================================================

FUNCTION PlaceStar(X AS LONG, Y AS LONG) AS LONG                      '// this puts those neat little navigation aids on the board
  LOCAL CalcedX AS LONG                                               '// holds the pixel offset of the star on our board
  LOCAL CalcedY AS LONG
  CalcedX = ((X - 1) * 20) + 18
  CalcedY = ((Y - 1) * 20) + 18
  GRAPHIC ELLIPSE (CalcedX, CalcedY)-(CalcedX + 5, CalcedY + 5), %BLACK, boardcolor, 0
END FUNCTION

'=====================================================================

FUNCTION count4(a AS STRING, c AS LONG) AS LONG                       '// used by the RECOUNT function - counts the "4-in-a-row" groups found
  REGISTER i AS LONG                                                  '// generic counter/pointer
  LOCAL p AS STRING                                                   '// holds the player number
  LOCAL buf AS STRING
  buf = a
  p = FORMAT$(ABS(c))                                                 '// turn the positive number into a string for quick comparison
  win5 = win5 + TALLY(buf, p + p + p + p + p)                         '// do we have a 5 match?
  IF c > 0 THEN                                                       '// positive numbers check for consecutive 4
    i = TALLY(buf, p + p + p + p)                                     '// count the number of groups
    FUNCTION = i                                                      '// return what we found
  ELSE                                                                '// check for open 4 pattern
    LOCAL t AS LONG                                                   '// track the total pairs we find
    LOCAL xpattern AS STRING                                          '// the pattern we are wanting
    t = 0                                                             '// start the count at zero
    '// Open 4 Patterns = xxxx, x_xxx, xx_xx, xxx_x, x_xx_x (with or without one stone at either end)
    xpattern = $SPC + p + p + p + p                                    '// the pattern to find
    GOSUB count4_work
    xpattern = p + $SPC + p + p + p                                    '// count the 4 open in the middle
    GOSUB count4_work
    xpattern = p + p + $SPC + p + p                                    '// count the 4 open in the middle
    GOSUB count4_work
    xpattern = p + p + p + $SPC + p                                    '// count the 4 open in the middle
    GOSUB count4_work
    xpattern = p + p + p + p + $SPC                                    '// count the consecutive 4 open at end
    GOSUB count4_work
    xpattern = p + $SPC + p + p + $SPC + p                             '// count the consecutive 4 open at end
    GOSUB count4_work
    FUNCTION = t                                            '// return what we found
  END IF
  EXIT FUNCTION
count4_work:
  i = INSTR(buf, xpattern)                                            '// find first pattern in buffer
  DO WHILE i > 0                                                      '// loop until we find all of these patterns
    INCR t                                                            '// count that we found this match
    buf = LEFT$(buf, i -1) + MID$(buf, i + LEN(xpattern) + 1)         '// remove the part we found so it won't be counted twice
    i = INSTR(buf, p)
  LOOP
  RETURN
END FUNCTION

'=====================================================================

FUNCTION recount(player AS LONG) AS LONG                              '// update the score based on the number of "4-groups" we find
  LOCAL i AS LONG
  REGISTER x AS LONG
  REGISTER y AS LONG
  REGISTER xx AS LONG
  REGISTER yy AS LONG
  LOCAL dx AS LONG
  LOCAL dy AS LONG
  LOCAL buf AS STRING
  LOCAL warn AS LONG                                                  '// flag to show that we have at least one open 4
  FOR i = 0 TO UBOUND(ts())                                           '// reset all temp scores
    ts(i) = 0                                                         '// start all temp scores at zero
  NEXT
  '// East-West check "-"
  buf = ""                                                            '// start with an empty buffer
  FOR y = 1 TO 19                                                     '// count the rows
    FOR x = 1 TO 19
      IF board(x, y) > 0 THEN
        buf = buf + FORMAT$(board(x, y))                              '// add this player's stone
      ELSE
        buf = buf + $SPC                                              '// nothing in this spot
      END IF
    NEXT
    buf = buf + $CR                                                   '// add an end of line marker
  NEXT
  GOSUB recount_work                                                  '// do the pattern checking work
  '// North-South check "|"
  buf = ""                                                            '// start with an empty buffer
  FOR x = 1 TO 19                                                     '// count the cols
    FOR y = 1 TO 19
      IF board(x, y) > 0 THEN
        buf = buf + FORMAT$(board(x, y))                              '// add this player's stone
      ELSE
        buf = buf + $SPC                                              '// nothing in this spot
      END IF
    NEXT
    buf = buf + $CR                                                   '// add an end of line marker
  NEXT
  GOSUB recount_work                                                  '// do the pattern checking work
  '// NorthWest-SouthEast check "\"
  buf = ""                                                            '// start with a blank buffer
  FOR y = 19 TO 1 STEP -1                                             '// walk up the side
    yy = y                                                            '// start at this row
    xx = 1                                                            '// always start in the first col
    DO WHILE yy < 20 AND xx < 20
      IF board(xx, yy) > 0 THEN
        buf = buf + FORMAT$(board(xx, yy))                            '// add this player's stone
      ELSE
        buf = buf + $SPC                                              '// nothing in this spot
      END IF
      INCR yy
      INCR xx
    LOOP
    buf = buf + $CR                                                   '// add end of line marker
  NEXT x
  FOR x = 2 TO 19                                                     '// walk across the top
    yy = 1                                                            '// always start at the first row
    xx = x                                                            '// start in this col
    DO WHILE yy < 20 AND xx < 20
      IF board(xx, yy) > 0 THEN
        buf = buf + FORMAT$(board(xx, yy))                            '// add this player's stone
      ELSE
        buf = buf + $SPC                                              '// nothing in this spot
      END IF
      INCR yy
      INCR xx
    LOOP
    buf = buf + $CR                                                   '// add end of line marker
  NEXT x
  GOSUB recount_work                                                  '// do the pattern checking work
  '// SouthWest-NorthEast check "/"
  buf = ""                                                            '// start with a blank buffer
  FOR y = 19 TO 1 STEP -1                                             '// walk up the side
    yy = y                                                            '// start at this row
    xx = 1                                                            '// always start in the first col
    DO WHILE yy < 20 AND xx < 20
      IF board(20 - xx, yy) > 0 THEN
        buf = buf + FORMAT$(board(20 - xx, yy))                       '// add this player's stone
      ELSE
        buf = buf + $SPC                                              '// nothing in this spot
      END IF
      INCR yy
      INCR xx
    LOOP
    buf = buf + $CR                                                   '// add end of line marker
  NEXT x
  FOR x = 2 TO 19                                                     '// walk across the top
    yy = 1                                                            '// always start at the first row
    xx = x                                                            '// start in this col
    DO WHILE yy < 20 AND xx < 20
      IF board(20 - xx, yy) > 0 THEN
        buf = buf + FORMAT$(board(20 - xx, yy))                       '// add this player's stone
      ELSE
        buf = buf + $SPC                                              '// nothing in this spot
      END IF
      INCR yy
      INCR xx
    LOOP
    buf = buf + $CR                                                   '// add end of line marker
  NEXT x
  GOSUB recount_work                                                  '// do the pattern checking work
  EXIT FUNCTION                                                       '// leave the function, we are done counting

recount_work:
  i = count4(buf, 1)                                                  '// find a match for player 1
  IF i > 0 THEN                                                       '// did we find anything?
    ts(1) = ts(1) + i                                                 '// add it to their score
  END IF
  i = count4(buf, 2)                                                  '// find a match for player 2
  IF i > 0 THEN                                                       '// did we find anything?
    ts(2) = ts(2) + i                                                 '// add it to their score
  END IF
  IF player < 0 THEN                                                  '// should we look for open 4 patterns too?
    warn = warn + count4(buf, player)                                 '// add to our open 4 total
  END IF
  RETURN
END FUNCTION

'=====================================================================

FUNCTION think(xx AS LONG, yy AS LONG) AS LONG                        '// this is where to do most of our thinking work
  STATIC thinking AS LONG                                             '// this var will always be available for every call
  IF thinking THEN                                                    '// are we already busy thinking? (user may be impatient)
    EXIT FUNCTION                                                     '// leave and let the first thought complete
  END IF
  thinking = -1                                                       '// flag to prevent recursive thinking
  LOCAL buf AS STRING                                                 '// local string buffer
  REGISTER x AS LONG                                                     '// local x counter
  REGISTER y AS LONG                                                     '// local y counter
  REGISTER i AS LONG                                                     '// flag and counter
  LOCAL p AS LONG                                                     '// the player number
  LOCAL win AS LONG                                                   '// counts the number of 5-in-a-row for this player
  i = 0                                                               '// start with nothing set
  p = board(xx, yy)                                                   '// remember this player's number
  '// Check for captures
  IF xx > 3 THEN                                                      '// West
    IF board(xx -1, yy) > -1 AND board(xx -1, yy) <> board(xx, yy) THEN '// there is a stone there, but it is different from our stone
      IF board(xx -2, yy) > -1 AND board(xx -1, yy) = board(xx -2,yy) THEN '// two of their stones in a row?
        IF board(xx -3, yy) > -1 AND board(xx, yy) = board(xx -3, yy) THEN '// our stones are on both side of their stones?
          capture(p) = capture(p) + 1                                 '// add one to the player's capture score
          score(p) = score(p) + 1                                     '// add one to the players score
          board(xx -1, yy) = %gone                                    '// make a shadow to show stone was removed
          board(xx -2, yy) = %gone
          INCR i
          history(UBOUND(history())) = "[" + TRIM$(history(UBOUND(history())), ANY "[]") + "]" '// add capture markers
        END IF
      END IF
    END IF
  END IF
  IF xx < 17 THEN                                                     '// East
    IF board(xx +1, yy) > -1 AND board(xx +1, yy) <> board(xx, yy) THEN '// there is a stone there, but it is different from our stone
      IF board(xx +2, yy) > -1 AND board(xx +1, yy) = board(xx +2,yy) THEN '// two of their stones in a row?
        IF board(xx +3, yy) > -1 AND board(xx, yy) = board(xx +3, yy) THEN '// our stones are on both side of their stones?
          capture(p) = capture(p) + 1                                 '// add one to the player's capture score
          score(p) = score(p) + 1                                     '// add one to the players score
          board(xx +1, yy) = %gone                                    '// make a shadow to show stone was removed
          board(xx +2, yy) = %gone
          INCR i
          history(UBOUND(history())) = "[" + TRIM$(history(UBOUND(history())), ANY "[]") + "]" '// add capture markers
        END IF
      END IF
    END IF
  END IF
  IF yy > 3 THEN                                                      '// North
    IF board(xx, yy -1) > -1 AND board(xx, yy -1) <> board(xx, yy) THEN '// there is a stone there, but it is different from our stone
      IF board(xx, yy -2) > -1 AND board(xx, yy -1) = board(xx,yy -2) THEN '// two of their stones in a row?
        IF board(xx, yy -3) > -1 AND board(xx, yy) = board(xx, yy -3) THEN '// our stones are on both side of their stones?
          capture(p) = capture(p) + 1                                 '// add one to the player's capture score
          score(p) = score(p) + 1                                     '// add one to the players score
          board(xx, yy -1) = %gone                                    '// make a shadow to show stone was removed
          board(xx, yy -2) = %gone
          INCR i
          history(UBOUND(history())) = "[" + TRIM$(history(UBOUND(history())), ANY "[]") + "]" '// add capture markers
        END IF
      END IF
    END IF
  END IF
  IF yy < 17 THEN                                                     '// South
    IF board(xx, yy +1) > -1 AND board(xx, yy +1) <> board(xx, yy) THEN '// there is a stone there, but it is different from our stone
      IF board(xx, yy +2) > -1 AND board(xx, yy +1) = board(xx,yy +2) THEN '// two of their stones in a row?
        IF board(xx, yy +3) > -1 AND board(xx, yy) = board(xx, yy +3) THEN '// our stones are on both side of their stones?
          capture(p) = capture(p) + 1                                 '// add one to the player's capture score
          score(p) = score(p) + 1                                     '// add one to the players score
          board(xx, yy +1) = %gone                                    '// make a shadow to show stone was removed
          board(xx, yy +2) = %gone
          INCR i
          history(UBOUND(history())) = "[" + TRIM$(history(UBOUND(history())), ANY "[]") + "]" '// add capture markers
        END IF
      END IF
    END IF
  END IF
  IF xx > 3 AND yy > 3 THEN                                           '// North West
    IF board(xx -1, yy -1) > -1 AND board(xx -1, yy -1) <> board(xx, yy) THEN '// there is a stone there, but it is different from our stone
      IF board(xx -2, yy -2) > -1 AND board(xx -1, yy -1) = board(xx -2, yy -2) THEN '// two of their stones in a row?
        IF board(xx -3, yy -3) > -1 AND board(xx, yy) = board(xx -3, yy -3) THEN '// our stones are on both side of their stones?
          capture(p) = capture(p) + 1                                 '// add one to the player's capture score
          score(p) = score(p) + 1                                     '// add one to the players score
          board(xx -1, yy -1) = %gone                                 '// make a shadow to show stone was removed
          board(xx -2, yy -2) = %gone
          INCR i
          history(UBOUND(history())) = "[" + TRIM$(history(UBOUND(history())), ANY "[]") + "]" '// add capture markers
        END IF
      END IF
    END IF
  END IF
  IF xx < 17 AND yy < 17 THEN                                         '// South East
    IF board(xx +1, yy +1) > -1 AND board(xx +1, yy +1) <> board(xx, yy) THEN '// there is a stone there, but it is different from our stone
      IF board(xx +2, yy +2) > -1 AND board(xx +1, yy +1) = board(xx +2, yy +2) THEN '// two of their stones in a row?
        IF board(xx +3, yy +3) > -1 AND board(xx, yy) = board(xx +3, yy +3) THEN '// our stones are on both side of their stones?
          capture(p) = capture(p) + 1                                 '// add one to the player's capture score
          score(p) = score(p) + 1                                     '// add one to the players score
          board(xx +1, yy +1) = %gone                                 '// make a shadow to show stone was removed
          board(xx +2, yy +2) = %gone
          INCR i
          history(UBOUND(history())) = "[" + TRIM$(history(UBOUND(history())), ANY "[]") + "]" '// add capture markers
        END IF
      END IF
    END IF
  END IF
  IF xx > 3 AND yy < 17 THEN                                          '// South West
    IF board(xx -1, yy +1) > -1 AND board(xx -1, yy +1) <> board(xx, yy) THEN '// there is a stone there, but it is different from our stone
      IF board(xx -2, yy +2) > -1 AND board(xx -1, yy +1) = board(xx -2, yy +2) THEN '// two of their stones in a row?
        IF board(xx -3, yy +3) > -1 AND board(xx, yy) = board(xx -3, yy +3) THEN '// our stones are on both side of their stones?
          capture(p) = capture(p) + 1                                 '// add one to the player's capture score
          score(p) = score(p) + 1                                     '// add one to the players score
          board(xx -1, yy +1) = %gone                                 '// make a shadow to show stone was removed
          board(xx -2, yy +2) = %gone
          INCR i
          history(UBOUND(history())) = "[" + TRIM$(history(UBOUND(history())), ANY "[]") + "]" '// add capture markers
        END IF
      END IF
    END IF
  END IF
  IF xx < 17 AND yy > 3 THEN                                          '// North East
    IF board(xx +1, yy -1) > -1 AND board(xx +1, yy -1) <> board(xx, yy) THEN '// there is a stone there, but it is different from our stone
      IF board(xx +2, yy -2) > -1 AND board(xx +1, yy -1) = board(xx +2, yy -2) THEN '// two of their stones in a row?
        IF board(xx +3, yy -3) > -1 AND board(xx, yy) = board(xx +3, yy -3) THEN '// our stones are on both side of their stones?
          capture(p) = capture(p) + 1                                 '// add one to the player's capture score
          score(p) = score(p) + 1                                     '// add one to the players score
          board(xx +1, yy -1) = %gone                                 '// make a shadow to show stone was removed
          board(xx +2, yy -2) = %gone
          INCR i
          history(UBOUND(history())) = "[" + TRIM$(history(UBOUND(history())), ANY "[]") + "]" '// add capture markers
        END IF
      END IF
    END IF
  END IF
  IF i > 0 THEN                                                       '// did something change?
    update_board                                                      '// show those changes
  END IF

  '// Did this stone make an open-4?

  '// East-West "-"
  i = 0                                                               '// set flag to show open4 count
  buf = ""                                                            '// start with an empty buffer
  FOR x = 1 TO 19                                                     '// walk this row
    IF board(x, yy) > 0 THEN                                          '// is there a stone here?
      buf = buf + FORMAT$(board(x, yy))                               '// add this player's stone
    ELSE
      buf = buf + $SPC                                                '// show nothing in this spot (might be a shadow from a previous stone)
    END IF
  NEXT
  buf = LEFT$(buf, xx + 6)                                            '// only keep 5 places after current stone
  IF xx > 5 THEN                                                      '// is current stone more than 4 from the edge?
    buf = MID$(buf, xx - 6)                                           '// keep only 5 places to the left
  END IF
  i = i + count4(buf, -board(xx,yy))                                  '// did this stone make an open 4?
  '// North-South "|"
  buf = ""                                                            '// start with an empty buffer
  FOR y = 1 TO 19                                                     '// walk this row
    IF board(xx, y) > 0 THEN
      buf = buf + FORMAT$(board(xx, y))                               '// add this player's stone
    ELSE
      buf = buf + $SPC                                                '// nothing in this spot
    END IF
  NEXT
  buf = LEFT$(buf, yy + 6)                                            '// only keep 5 places after current stone
  IF yy > 5 THEN                                                      '// is current stone more than 4 from the edge?
    buf = MID$(buf, yy - 6)                                           '// keep only 5 places to the left
  END IF
  i = i + count4(buf, -board(xx,yy))                                  '// did this stone make an open 4?
  '// NorthWest-SouthEast "\"
  buf = ""                                                            '// start with an empty buffer
  IF xx > yy THEN
    x = 1 + xx - yy
    y = 1
  ELSEIF yy > xx THEN
    y = 1 + yy - xx
    x = 1
  ELSE                                                                '// they both are even = center of board
    x = 1
    y = 1
  END IF
  p = 0                                                               '// start our pointer at
  DO WHILE y < 20 AND x < 20
    IF xx = x THEN                                                    '// did we reach our starting place on the diag?
      p = LEN(buf)                                                    '// remember our place in the buffer
    END IF
    IF board(x, y) > 0 THEN
      buf = buf + FORMAT$(board(x, y))                                '// add this player's stone
    ELSE
      buf = buf + $SPC                                                '// nothing in this spot
    END IF
    INCR y
    INCR x
  LOOP
  buf = LEFT$(buf, p + 6)                                             '// only keep 5 places after current stone
  IF p > 5 THEN                                                       '// is current stone more than 4 from the edge?
    buf = MID$(buf, p - 6)                                            '// keep only 5 places to the left
  END IF
  i = i + count4(buf, -board(xx,yy))                                  '// did this stone make an open 4?
  '// SouthWest-NorthEast "/"
  buf = ""                                                            '// start with an empty buffer
  x = xx
  y = yy
  DO WHILE x < 20 AND y > 0
    DECR y
    INCR x
  LOOP
  p = 0
  DO WHILE y < 20 AND x > 0
    IF xx = x THEN                                                    '// did we reach our starting place on the diag?
      p = LEN(buf)                                                    '// remember our place in the buffer
    END IF
    IF board(x, y) > 0 THEN                                           '// is there a stone in this place?
      buf = buf + FORMAT$(board(x, y))                                '// add this player's stone
    ELSE
      buf = buf + $SPC                                                '// show this spot as empty
    END IF
    INCR y
    DECR x
  LOOP
  buf = LEFT$(buf, p + 6)                                             '// only keep 5 places after current stone
  IF p > 5 THEN                                                       '// is current stone more than 4 from the edge?
    buf = MID$(buf, p - 6)                                            '// keep only 5 places to the left
  END IF
  i = i + count4(buf, -board(xx,yy))                                  '// did this stone make an open 4?
  IF i > 0 THEN                                                       '// is there at least one open 4 for this player?
    history(UBOUND(history())) = RTRIM$(history(UBOUND(history())), "!") + "!" '// add open4 marker
  END IF
  i = recount(board(xx,yy))                                           '// update scores and find any open 4
  IF win5 THEN                                                        '// did this player get a 5-in-a-row?
    score(board(xx,yy)) = score(board(xx,yy)) + 5                     '// add 5 to their score
    history(UBOUND(history())) = "*" + RTRIM$(history(UBOUND(history())), "!") + "*" '// add winning markers
  END IF
  IF capture(1) > 3 THEN
    CONTROL SET COLOR ghDlg, %IDC_CAPTURE1_LABEL, %BLACK, %YELLOW
  END IF
  CONTROL SET TEXT ghDlg, %IDC_CAPTURE1_LABEL, FORMAT$(capture(1))
  IF capture(2) > 3 THEN
    CONTROL SET COLOR ghDlg, %IDC_CAPTURE2_LABEL, %BLACK, %YELLOW
  END IF
  CONTROL SET TEXT ghDlg, %IDC_CAPTURE2_LABEL, FORMAT$(capture(2))
  CONTROL SET TEXT ghDlg, %IDC_SCORE1_LABEL, FORMAT$(score(1) + ts(1))
  CONTROL SET TEXT ghDlg, %IDC_SCORE2_LABEL, FORMAT$(score(2) + ts(2))
  p = UBOUND(history())                                               '// find the total number of history actions
  x = 0                                                               '// start the line counter at zero
  buf = ""                                                            '// start with a blank buffer
  FOR i = 0 TO p STEP 2                                               '// build the history log
    IF history(i) = "" THEN                                           '// is this a blank entry?
      EXIT FOR                                                        '// leave this loop
    END IF
    INCR x                                                            '// add one to the line counter
    buf = FORMAT$(x) + ":" + $TAB + history(i) + IIF$(i < p, $TAB + history(i +1), "") + $CRLF + buf '// build the line
  NEXT
  CONTROL SET TEXT ghDlg, %IDC_HISTORY, buf                           '// show the action history log

  '// Check for winning condition
  IF capture(1) > 4 OR capture(2) > 4 OR win5 THEN
    gameover = -1                                                     '// set game-over flag to true
  END IF
  IF gameover THEN
    IF score(1) + ts(1) > score(2) + ts(2) THEN                       '// player 1 wins with high score
      winner = 1                                                      '// remember the winner
      CONTROL SET TEXT ghDlg, %IDC_STATUS, plang(%lang_wins, 1, 0)
    ELSEIF score(1) + ts(1) < score(2) + ts(2) THEN                   '// player 2 wins with high score
      winner = 2
      CONTROL SET TEXT ghDlg, %IDC_STATUS, plang(%lang_wins, 2, 0)
    ELSE                                                              '// tie score
      winner = -1
      CONTROL SET TEXT ghDlg, %IDC_STATUS, lang(%lang_tie)
    END IF
    update_board                                                      '// show winner
    IF win5 THEN
      MSGBOX plang(%lang_won5, board(xx,yy), 0), %MB_OK OR %MB_ICONINFORMATION OR %MB_TASKMODAL, lang(%lang_congrats)
    ELSEIF capture(1) > 4 THEN
      MSGBOX plang(%lang_won_capts, 1, capture(1)), %MB_OK OR %MB_ICONINFORMATION OR %MB_TASKMODAL, lang(%lang_congrats)
    ELSEIF capture(2) > 4 THEN
      MSGBOX plang(%lang_won_capts, 2, capture(2)), %MB_OK OR %MB_ICONINFORMATION OR %MB_TASKMODAL, lang(%lang_congrats)
    END IF
  END IF
  thinking = 0                                                        '// turn flag off to allow more thinking to happen
END FUNCTION
